/**
 * <a href="http://www.openolat.org">
 * OpenOLAT - Online Learning and Training</a><br>
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); <br>
 * you may not use this file except in compliance with the License.<br>
 * You may obtain a copy of the License at the
 * <a href="http://www.apache.org/licenses/LICENSE-2.0">Apache homepage</a>
 * <p>
 * Unless required by applicable law or agreed to in writing,<br>
 * software distributed under the License is distributed on an "AS IS" BASIS, <br>
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
 * See the License for the specific language governing permissions and <br>
 * limitations under the License.
 * <p>
 * Initial code contributed and copyrighted by<br>
 * frentix GmbH, http://www.frentix.com
 * <p>
 */
package org.olat.ims.qti21.ui.editor;

import org.olat.core.gui.UserRequest;
import org.olat.core.gui.components.form.flexible.FormItemContainer;
import org.olat.core.gui.components.form.flexible.elements.TextElement;
import org.olat.core.gui.components.form.flexible.impl.FormBasicController;
import org.olat.core.gui.components.form.flexible.impl.FormLayoutContainer;
import org.olat.core.gui.control.Controller;
import org.olat.core.gui.control.Event;
import org.olat.core.gui.control.WindowControl;
import org.olat.ims.qti21.model.xml.ManifestMetadataBuilder;

/**
 * 
 * Initial date: 23.02.2016<br>
 * @author srosse, stephane.rosse@frentix.com, http://www.frentix.com
 *
 */
public class MetadataEditorController extends FormBasicController {
	
	private TextElement titleEl, descriptionEl;
	//keywords, language
	
	private final ManifestMetadataBuilder metadataBuilder;
	
	public MetadataEditorController(UserRequest ureq, WindowControl wControl, ManifestMetadataBuilder metadataBuilder) {
		super(ureq, wControl);
		this.metadataBuilder = metadataBuilder;
		initForm(ureq);
	}

	@Override
	protected void initForm(FormItemContainer formLayout, Controller listener, UserRequest ureq) {
		String title = metadataBuilder.getTitle();
		titleEl = uifactory.addTextElement("title", "form.metadata.title", 256, title, formLayout);
		
		String description = metadataBuilder.getDescription();
		descriptionEl = uifactory.addTextAreaElement("description", "form.metadata.description", 4000, 4, 60, true, description, formLayout);
		
		// Submit Button
		FormLayoutContainer buttonsContainer = FormLayoutContainer.createButtonLayout("buttons", getTranslator());
		buttonsContainer.setRootForm(mainForm);
		formLayout.add(buttonsContainer);
		uifactory.addFormSubmitButton("submit", buttonsContainer);
	}

	@Override
	protected void doDispose() {
		//
	}
	
	@Override
	protected void formOK(UserRequest ureq) {
		metadataBuilder.setTitle(titleEl.getValue(), "en");
		metadataBuilder.setDescription(descriptionEl.getValue(), "en");
		fireEvent(ureq, Event.CHANGED_EVENT);
	}
}
