/**
 * <a href="http://www.openolat.org">
 * OpenOLAT - Online Learning and Training</a><br>
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); <br>
 * you may not use this file except in compliance with the License.<br>
 * You may obtain a copy of the License at the
 * <a href="http://www.apache.org/licenses/LICENSE-2.0">Apache homepage</a>
 * <p>
 * Unless required by applicable law or agreed to in writing,<br>
 * software distributed under the License is distributed on an "AS IS" BASIS, <br>
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
 * See the License for the specific language governing permissions and <br>
 * limitations under the License.
 * <p>
 * Initial code contributed and copyrighted by<br>
 * frentix GmbH, http://www.frentix.com
 * <p>
 */
package org.olat.modules.lecture.model;

import org.olat.modules.lecture.LectureBlock;
import org.olat.modules.lecture.LectureBlockStatus;
import org.olat.modules.lecture.LectureModule;
import org.olat.modules.lecture.LectureRollCallStatus;
import org.olat.modules.lecture.RollCallSecurityCallback;

/**
 * 
 * Initial date: 11 avr. 2017<br>
 * @author srosse, stephane.rosse@frentix.com, http://www.frentix.com
 *
 */
public class RollCallSecurityCallbackImpl implements RollCallSecurityCallback {

	private final boolean repoAdmin;
	private final boolean teacher;
	private LectureBlock lectureBlock;
	private final LectureModule lectureModule;
	
	public RollCallSecurityCallbackImpl(boolean repoAdmin, boolean teacher, LectureBlock lectureBlock, LectureModule lectureModule) {
		this.repoAdmin = repoAdmin;
		this.teacher = teacher;
		this.lectureBlock = lectureBlock;
		this.lectureModule = lectureModule;
	}
	
	@Override
	public void updateLectureBlock(LectureBlock block) {
		lectureBlock = block;
	}

	@Override
	public boolean canEdit() {
		if(isClosed() || isCancelled()) return false;
		return (repoAdmin || teacher) && isBlockEditable();
	}

	@Override
	public boolean canViewAuthorizedAbsences() {
		boolean autorizedAbsenceAllowed = lectureModule.isAuthorizedAbsenceEnabled();
		if(autorizedAbsenceAllowed) {
			if(repoAdmin) {
				return true;
			}
			return !isBlockEditable() && lectureModule.isTeacherCanAuthorizedAbsence();
		}
		return false;
	}

	@Override
	public boolean canEditAuthorizedAbsences() {
		if(isClosed() || isCancelled()) return false;
		boolean autorizedAbsenceAllowed = lectureModule.isAuthorizedAbsenceEnabled();
		if(autorizedAbsenceAllowed) {
			if(repoAdmin) {
				return isBlockEditable();
			}
			return isBlockEditable() && lectureModule.isTeacherCanAuthorizedAbsence();
		}
		return false;
	}

	@Override
	public boolean canEditAbsences() {
		if(isClosed() || isCancelled()) return false;
		return repoAdmin || (teacher && isBlockEditable());
	}
	
	@Override
	public boolean canReopen() {
		return repoAdmin && (isClosed() || isCancelled());
	}

	private boolean isBlockEditable() {
		return lectureBlock.getRollCallStatus() == LectureRollCallStatus.open
			|| lectureBlock.getRollCallStatus() == LectureRollCallStatus.reopen;
	}
	
	private boolean isCancelled() {
		return lectureBlock.getStatus() == LectureBlockStatus.cancelled;
	}
	
	private boolean isClosed() {
		return lectureBlock.getRollCallStatus() == LectureRollCallStatus.closed
			|| lectureBlock.getRollCallStatus() == LectureRollCallStatus.autoclosed;
	}
}
